@@ -97,6 +97,17 @@ class StructureContainer(GenericJob, HasStructure):
         self._container.add_structure(struct)
         return struct
 
+    def add_structure(self, structure: Atoms, identifier: str = None, **kwargs):
+        """
+        Add a new structure.
+
+        Args:
+            structure (:class:`~.Atoms`): structure to add
+            identifier (str, optional): optional identifier for the structure
+            **kwargs: passed through to the underlying :meth:`.StructureStorage.add_structure`
+        """
+        self._container.add_structure(structure, identifier=identifier, **kwargs)
+
     def run_static(self):
         self.status.finished = True
 
@@ -154,3 +165,10 @@ class StructureContainer(GenericJob, HasStructure):
         else:
             super().from_hdf(hdf=hdf, group_name=group_name)
             self._container.from_hdf(hdf=self.project_hdf5, group_name="structures")
+
+    @property
+    def plot(self):
+        """
+        Accessor for :class:`~.StructurePlots` instance using these structures.
+        """
+        return self._container.plot
