@@ -8,6 +8,7 @@ from __future__ import print_function
 import sys
 import copy
 import numpy as np
+from numbers import Integral
 from collections import OrderedDict
 from collections.abc import Sequence
 
@@ -172,7 +173,10 @@ class SparseList(object):
                 yield val
 
     def __getitem__(self, item):
-        if isinstance(item, (int, np.integer)):
+        if isinstance(item, Integral):
+            if item > len(self):
+                raise IndexError
+            item = item % len(self)
             if item in self._dict:
                 return self._dict[item]
             return self._default
@@ -183,13 +187,15 @@ class SparseList(object):
             if len(item) == 0:
                 ind_list = []
             else:
-                if isinstance(item[0], (int, np.integer)):
+                # for some reason isinstance(True, int) == True, so check for bools first explicitly
+                if isinstance(item[0], (bool, np.bool_)):
+                    if len(item) != len(self):
+                        raise IndexError(
+                            "Length of boolean index does not match indexed list!"
+                        )
+                    ind_list = np.argwhere(item).flatten()
+                elif isinstance(item[0], Integral):
                     ind_list = item
-                elif isinstance(item[0], (bool, np.bool_)):
-                    ind_list = []
-                    for i, bo in enumerate(item):
-                        if bo:
-                            ind_list.append(i)
         else:
             raise ValueError("Unknown item type: " + str(type(item)))
         sliced_dict = {
@@ -199,9 +205,10 @@ class SparseList(object):
         return self.__class__(sliced_dict, default=self._default, length=len(ind_list))
 
     def __setitem__(self, key, value):
-        if isinstance(key, (int, np.integer)):
+        if isinstance(key, Integral):
             if key > len(self):
                 raise IndexError
+            key = key % len(self)
             self._dict[key] = value
             return
         elif isinstance(key, slice):
@@ -209,8 +216,14 @@ class SparseList(object):
 
         if max(key) > self._length:
             raise IndexError
-        for i in key:
-            self._dict[i] = value
+
+        keys = list(key)
+        if isinstance(keys[0], (bool, np.bool_)):
+            for i in np.argwhere(keys).flatten():
+                self._dict[i] = value
+        else:
+            for i in key:
+                self._dict[i] = value
 
     def __delitem__(self, key):
         # programmed for simplicity, not for performance
@@ -238,17 +251,18 @@ class SparseList(object):
         return new_list
 
     def __mul__(self, other):
-        if not isinstance(other, (int, np.integer)):
+        if not isinstance(other, Integral):
             raise ValueError("Multiplication defined only for SparseArray*integers")
-        overall_list = other * np.arange(len(self)).tolist()
-        new_dic = dict()
-        for k in self.keys():
-            for val in np.argwhere(np.array(overall_list) == k).flatten():
-                new_dic[val] = self[k]
-        return self.__class__(new_dic, default=self._default, length=other * len(self))
+        new_dict = {}
+        length = len(self)
+        for rep in range(other):
+            offset = rep * length
+            for k in self.keys():
+                new_dict[k + offset] = self[k]
+        return self.__class__(new_dict, default=self._default, length=other * length)
 
     def __rmul__(self, other):
-        if isinstance(other, int):
+        if isinstance(other, Integral):
             return self * other
 
     def __str__(self):
@@ -421,7 +435,7 @@ class SparseArray(object):
 
     def __getitem__(self, item):
         new_dict = {}
-        if isinstance(item, int):
+        if isinstance(item, Integral):
             for key, value in self._lists.items():
                 if value[item] is not None:
                     new_dict[key] = value[item]
@@ -534,7 +548,7 @@ class SparseArray(object):
         return new_array
 
     def __mul__(self, other):
-        if not isinstance(other, int):
+        if not isinstance(other, Integral):
             raise ValueError(
                 "Multiplication with SparseMatrix only implemented for integers"
             )
@@ -546,7 +560,7 @@ class SparseArray(object):
         return new_array
 
     def __rmul__(self, other):
-        if isinstance(other, int):
+        if isinstance(other, Integral):
             return self * other
 
     def add_tag(self, *args, **qwargs):
