@@ -51,6 +51,7 @@ class ElectronicStructure(object):
         self.n_spins = 1
         self._structure = None
         self._orbital_dict = None
+        self._output_dict = {}
 
     def add_kpoint(self, value, weight):
         """
@@ -469,6 +470,9 @@ class ElectronicStructure(object):
         """
         self._grand_dos_matrix = val
 
+    def __getitem__(self, item):
+        return self._output_dict[item]
+
     def to_hdf(self, hdf, group_name="electronic_structure"):
         """
         Store the object to hdf5 file
@@ -533,6 +537,7 @@ class ElectronicStructure(object):
                         self.grand_dos_matrix = h_dos["grand_dos_matrix"]
                     if "resolved_densities" in nodes:
                         self.resolved_densities = h_dos["resolved_densities"]
+                self._output_dict = h_es.copy()
             self.generate_from_matrices()
 
     def to_hdf_old(self, hdf, group_name="electronic_structure"):
@@ -589,6 +594,7 @@ class ElectronicStructure(object):
                 self.grand_dos_matrix = h_es["grand_dos_matrix"]
             if "resolved_densities" in nodes:
                 self.resolved_densities = h_es["resolved_densities"]
+            self._output_dict = h_es.copy()
         self.generate_from_matrices()
 
     def generate_from_matrices(self):
@@ -672,7 +678,13 @@ class ElectronicStructure(object):
             import matplotlib.pyplot as plt
         for spin, eigenvalues in enumerate(self.eigenvalues):
             arg = np.argsort(eigenvalues)
-            plt.plot(eigenvalues[arg], self.occupancies[spin][arg], "-o", label="spin:{}".format(spin), linewidth=2)
+            plt.plot(
+                eigenvalues[arg],
+                self.occupancies[spin][arg],
+                "-o",
+                label="spin:{}".format(spin),
+                linewidth=2,
+            )
         plt.legend()
         plt.axvline(self.efermi, linewidth=2.0, linestyle="dashed", color="black")
         plt.xlabel("Eigen value (eV)")
@@ -698,17 +710,25 @@ class ElectronicStructure(object):
         output_string = list()
         output_string.append("ElectronicStructure Instance")
         output_string.append("----------------------------")
-        output_string.append("Number of spin channels: {}".format(len(self.eigenvalue_matrix)))
+        output_string.append(
+            "Number of spin channels: {}".format(len(self.eigenvalue_matrix))
+        )
         output_string.append("Number of k-points: {}".format(len(self.kpoints)))
-        output_string.append("Number of bands: {}".format(len(self.kpoints[0].bands[0])))
+        output_string.append(
+            "Number of bands: {}".format(len(self.kpoints[0].bands[0]))
+        )
         try:
             for spin, is_metal in enumerate(self.is_metal):
                 if is_metal:
-                    output_string.append("spin {}:".format(spin) + " Is a metal: {}".format(is_metal))
+                    output_string.append(
+                        "spin {}:".format(spin) + " Is a metal: {}".format(is_metal)
+                    )
                 else:
-                    output_string.append("spin {}:".format(spin) + " Is a metal: "
-                                                                   "{}".format(is_metal) + " Band gap (ev) "
-                                                                                           "{}".format(self.eg[spin]))
+                    output_string.append(
+                        "spin {}:".format(spin) + " Is a metal: "
+                        "{}".format(is_metal) + " Band gap (ev) "
+                        "{}".format(self.eg[spin])
+                    )
         except ValueError:
             pass
         return "\n".join(output_string)
