@@ -525,7 +525,6 @@ class LammpsControl(GenericParameters):
             if langevin:
                 warnings.warn("Temperature not set; Langevin ignored.")
             fix_ensemble_str = "all nve"
-            initial_temperature = 0
 
         if tloop is not None:
             fix_ensemble_str += " tloop " + str(tloop)
@@ -539,7 +538,7 @@ class LammpsControl(GenericParameters):
             append_if_not_present=True,
         )
 
-        if initial_temperature > 0:
+        if initial_temperature is not None and initial_temperature > 0:
             self.set_initial_velocity(
                 temperature=initial_temperature,
                 seed=seed,
